/*
 * Copyright (C) 2012 Chris Burdess <dog@gnu.org>
 *
 * Gnome Shell Biff is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Gnome Shell Biff is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Gnome Documents; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

const St = imports.gi.St;
const Main = imports.ui.main;
const GConf = imports.gi.GConf;
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Mainloop = imports.mainloop;
const Utils = imports.misc.util;
const GnomeKeyring = imports.gi.GnomeKeyring;
const EDataServer = imports.gi.EDataServer;
const Extension = imports.ui.extensionSystem.extensions["biff@bluezoo.org"];
const Imap = Extension.imap;

let gconfclient, check_interval, checking;
let button, extension_path;
let check_mail_pid;
let icon_read, icon_unread, label;

function open_mua() {
    try {
        let appinfo = Gio.app_info_get_default_for_type("x-scheme-handler/mailto", false);
        let success = appinfo.launch([], null);
        if (!success) {
            global.log("Error launching default MUA");
        }
    } catch (err) {
        global.log("Can't start default MUA: "+err.message);
    }
}

function check_mail() {
    if (checking) {
        return;
    }
    checking = true;
    try {
        let list = get_mail_accounts();
        for (let i = 0; i < list.length; i++) {
            let url = list[i];
            // split url into components
            let si = url.indexOf(":");
            let scheme = url.substring(0, si);
            si += 3;
            let ei = url.indexOf("/", si);
            let mailbox = url.substring(ei + 1);
            let ui = url.lastIndexOf("@", ei);
            let host = url.substring(ui + 1, ei);
            let username = url.substring(si, ui);
            check_account(scheme, host, username, mailbox);
        }
    } catch (err) {
        global.log("Can't check mail: "+err.message);
    }
    checking = false;
    return null;
}

function get_mail_accounts() {
    try {
        let ret = new Array();
        let list = gconfclient.get("/apps/evolution/mail/accounts").get_list(GConf.VALUE_STRING);
        for (let i = 0; i < list.length; i++) {
            // TODO reinstate this when EDataServer's e_account_get_string
            // works
            //let account = EDataServer.Account.new_from_xml(list[i].get_string());
            //let url = account.get_string(6); // E_ACCOUNT_SOURCE_URL
            let entry = list[i].get_string();
            let si = entry.indexOf("<url>");
            let ei = entry.indexOf("</url>", si);
            let url = entry.substring(si + 5, ei);
            let sci = url.indexOf(';');
            let params = "";
            if (sci != -1) {
                params = url.substring(sci + 1);
                url = url.substring(0, sci);
            }
            if (url.substr(0, 5) === "imap:" && params.indexOf("security-method=ssl-on-alternate-port") != -1) {
                url = "imaps:" + url.substring(5);
            }
            ret.push(url);
        }
        return ret;
    } catch (err) {
        global.log("Error retrieving mail accounts: "+err.message);
    }
}

function check_account(scheme, host, username, mailbox) {
    let port = 143;
    if (scheme == "imaps") {
        port = 993;
    }
    let di = host.indexOf(":");
    if (di != -1) {
        port = parseInt(host.substring(di + 1));
        host = host.substring(0, di);
    }
    let password = null;
    let ui = username.indexOf(":");
    if (ui != -1) {
        password = GLib.uri_unescape_string(username.substring(ui + 1), "");
        username = username.substring(0, ui);
    }
    username = GLib.uri_unescape_string(username, "");
    if (password == null) {
        password = get_password(scheme, host, username);
    }
    if (password == null) {
        if (scheme == "imaps") {
            password = get_password("imap", host, username);
            if (password == null) { 
                password = get_password("imapx", host, username);
            }
        } else if (scheme == "imap") {
            password = get_password("imapx", host, username);
        }
    }
    if (password == null) {
        // TODO ask for password if none found and store it
        global.log("Can't retrieve password for "+username+" on "+host);
        return;
    }
    if (scheme == "imap" || scheme == "imaps") {
        let imap = new Imap.IMAPConnection(host, port, username, password, (scheme === "imaps"), mailbox, update_unread_count);
        imap.connect();
    } else {
        // TODO maybe others
        global.log("Unknown scheme "+scheme);
    }
}

function get_password(scheme, host, username) {
    let password = null;
    let type = 1; /* NETWORK_PASSWORD */
    let attributes = GnomeKeyring.attribute_list_new();
    GnomeKeyring.attribute_list_append_string(attributes, "protocol", scheme);
    GnomeKeyring.attribute_list_append_string(attributes, "server", host);
    GnomeKeyring.attribute_list_append_string(attributes, "user", username);
    let [result, found] = GnomeKeyring.find_items_sync(type, attributes);
    if (result == 0 /* OK */) {
        for (let i = 0; i < found.length; i++) {
            password = found[i].secret;
        }
    }
    GnomeKeyring.attribute_list_free(attributes);
    return password;
}

function update_unread_count(account, value) {
    //global.log("update_unread_count "+account+"="+value);
    try {
        let total = value;
        // GConfClient.set_list doesn't work, use string instead
        let countsval = gconfclient.get("/apps/gnome-buff/counts-string");
        let counts = null;
        let acc = account + "=" + value;
        if (countsval != null) {
            counts = countsval.get_string();
        }
        if (counts == null) {
            counts = "";
        }
        let tokens = counts.split(",");
        for (let i = 0; i < tokens.length; i++) {
            let line = tokens[i].trim();
            if (line.length > 0) {
                let ei = line.lastIndexOf("=");
                let head = line.substring(0, ei);
                let tail = line.substring(ei + 1);
                if (head === account) {
                } else {
                    total = total + parseInt(tail);
                    acc = acc + "," + line;
                }
            }
        }
        gconfclient.set_string("/apps/gnome-buff/counts-string", acc);
        set_message_count(total);
    } catch (err) {
        global.log("Error updating new message count list: "+err.message);
    }
}

function init(meta) {
    extension_path = meta.path;
    checking = false;
    check_interval = 20;
    gconfclient = GConf.Client.get_default();
    let cci = gconfclient.get("/apps/gnome-biff/check-interval");
    if (cci != null) {
        check_interval = cci.get_int();
        if (check_interval < 10) {
            check_interval = 10; // don't invoke check_mail too frequently
        }
    }
    _init();
}

function enable() {
    Main.panel._rightBox.insert_actor(button, 0);
    check_mail();
    check_mail_pid = GLib.timeout_add_seconds(0, check_interval, check_mail);
}

function disable() {
    Main.panel._rightBox.remove_actor(button);
    if (check_mail_pid != null) {
        Mainloop.source_remove(check_mail_pid);
        check_mail_pid = null;
    }
}

// TODO move this to a separate prototype

function _init() {
    button = new St.Bin({ style_class: 'panel-button', reactive: true, can_focus: true, x_fill:true, y_fill: false, track_hover:true });
    icon_read = new St.Icon({ icon_name: 'mail-read-symbolic', icon_size: 16, icon_type: St.IconType.SYMBOLIC, style_class: 'system_status_icon' });
    icon_unread = new St.Icon({ icon_name: 'mail-unread-symbolic', icon_size: 16, icon_type: St.IconType.SYMBOLIC, style_class: 'system_status_icon' });
    icon_unread.hide();
    label = new St.Label({ style_class: 'button_text', text: '...' });
    let box = new St.BoxLayout();
    box.insert_actor(this.icon_read, 1);
    box.insert_actor(this.icon_unread, 1);
    box.insert_actor(this.label, 2);
    button.set_child(box);
    button.connect('button-press-event', open_mua);
}

function set_message_count(count) {
    if (count > 0) {
        label.text = count.toString();
        icon_read.hide();
        icon_unread.show();
        label.show();
    } else {
        icon_unread.hide();
        icon_read.show();
        label.hide();
    }
}

