/*
 * Copyright (C) 2012 Chris Burdess <dog@gnu.org>
 *
 * Gnome Shell Biff is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Gnome Shell Biff is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Gnome Documents; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

const GLib = imports.gi.GLib;

function CramMD5(username, password) {
    this.init(username, password);
}


CramMD5.prototype = {

init : function(username, password) {
    this.username = new Uint8Array(username);
    this.password = new Uint8Array(password);
    this.complete = false;
},

get_name : function() {
    return "CRAM-MD5";
},

is_plaintext : function() {
    return false;
},

has_initial_response : function() {
    return false;
},

evaluate_challenge : function(challenge) {
    let digest = this.hmac_md5(this.password, challenge);
    let ret = new Uint8Array(this.username.length + digest.length + 1);
    ret.set(this.username, 0);
    ret[this.username.length] = 0x20;
    ret.set(digest, this.username.length + 1);
    this.complete = true;
    return ret;
},

is_complete : function() {
    return this.complete;
},

hmac_md5 : function(key, text) {
    let k_ipad = new Uint8Array(64);
    let k_opad = new Uint8Array(64);
    let md5 = GLib.Checksum.new(0);
    let digest;
    if (key.length > 64) {
        md5.update(key, key.length);
        key = md5.get_digest();
    }
    for (let i = 0; i < 64; i++) {
        let val = 0;
        if (i < key.length) {
            val = key[i];
        }
        k_ipad[i] = val ^ 0x36;
        k_opad[i] = val ^ 0x5c;
    }
    md5.reset();
    md5.update(k_ipad, 64);
    md5.update(text, text.length);
    digest = md5.get_digest();
    md5.reset();
    md5.update(k_opad, 64);
    md5.update(digest, digest.length);
    digest = md5.get_digest();
    return digest;
}

};

function Plain(username, password) {
    this.init(username, password);
}

Plain.prototype = {

init : function(username, password) {
    this.username = new Uint8Array(username);
    this.password = new Uint8Array(password);
    this.complete = false;
},

get_name : function() {
    return "PLAIN";
},

is_plaintext : function() {
    return true;
},

has_initial_response : function() {
    return true;
},

evaluate_challenge : function(challenge) {
    let ret = new Uint8Array((this.username.length * 2) + this.password.length + 2);
    ret.set(this.username, 0);
    ret.set(this.username, this.username.length + 1);
    ret.set(this.password, (this.username.length * 2) + 2);
    this.complete = true;
    return ret;
},

is_complete : function() {
    return this.complete;
}

};
